@@ -6,7 +6,7 @@
 #ifndef _IP_VS_H
 #define _IP_VS_H
 
-#define IP_VS_VERSION_CODE            0x000003
+#define IP_VS_VERSION_CODE            0x000004
 #define NVERSION(version)                       \
         (version >> 16) & 0xFF,                 \
         (version >> 8) & 0xFF,                  \
@@ -189,8 +189,6 @@ enum {
 	IP_VS_S_LAST
 };
 
-struct ip_vs_dest;
-struct ip_vs_scheduler;
 
 struct ip_vs_timeout_table {
 	atomic_t refcnt;
@@ -214,8 +212,8 @@ union ip_vs_tphdr {
  */
 struct ip_vs_seq {
         __u32           init_seq;       /* Add delta from this seq */
-        short           delta;          /* Delta in sequence numbers */
-        short           previous_delta; /* Delta in sequence numbers
+        __u32           delta;          /* Delta in sequence numbers */
+        __u32           previous_delta; /* Delta in sequence numbers
                                            before last resized pkt */
 };
 
@@ -228,22 +226,25 @@ struct ip_vs_conn {
 	atomic_t                refcnt;         /* reference count */
 	struct timer_list       timer;          /* Expiration timer */
 	unsigned long           timeout;        /* timeout */
-	__u16                   protocol;       /* Which protocol (TCP/UDP) */
-        __u32                   caddr;          /* client address & port */
+        __u32                   caddr;          /* client address */
+        __u32                   vaddr;          /* virtual address */
+        __u32                   daddr;          /* destination address */
         __u16                   cport;
-        __u32                   vaddr;          /* VS address & port */
         __u16                   vport;
-        __u32                   daddr;          /* dest address & port */
         __u16                   dport;
-	struct ip_vs_app        *app;           /* bound ip_vs_app object */
-	void                    *app_data;      /* Application private data */
+	__u16                   protocol;       /* Which protocol (TCP/UDP) */
 	struct ip_vs_conn       *control;       /* Master control connection */
 	atomic_t                n_control;      /* Number of controlled ones */
-	unsigned                flags;          /* status flags */
-	unsigned                state;          /* state info */
+	__u16                   flags;          /* status flags */
+	__u16                   state;          /* state info */
 	struct ip_vs_timeout_table *timeout_table;
 	struct ip_vs_dest       *dest;          /* real server */
         atomic_t                in_pkts;        /* incoming packet counter */
+        /* Note: we can group the following members into a structure,
+           in order to save more space, and the following members are
+           only used in VS/NAT anyway */
+	struct ip_vs_app        *app;           /* bound ip_vs_app object */
+	void                    *app_data;      /* Application private data */
         struct ip_vs_seq        in_seq;         /* incoming seq. struct */
         struct ip_vs_seq        out_seq;        /* outgoing seq. struct */
 };
@@ -260,7 +261,7 @@ struct ip_vs_service {
 	__u16			protocol; /* which protocol (TCP/UDP) */
 	__u32			addr;	  /* IP address for virtual service */
 	__u16			port;	  /* port number for the service */
-	__u32			fwmark;   /* firewall mark of the service */
+	unsigned long		fwmark;   /* firewall mark of the service */
 	unsigned		flags;	  /* service status flags */
 	unsigned		timeout;  /* persistent timeout in ticks */
 	__u32			netmask;  /* grouping granularity */
@@ -292,7 +293,7 @@ struct ip_vs_dest {
 	__u16			protocol; /* which protocol (TCP/UDP) */
 	__u32			vaddr;	  /* IP address for virtual service */
 	__u16			vport;	  /* port number for the service */
-	__u32			vfwmark;  /* firewall mark of the service */
+	unsigned long		vfwmark;  /* firewall mark of the service */
 };
 
 
@@ -494,8 +495,9 @@ extern struct ip_vs_scheduler *ip_vs_lookup_scheduler(const char *sched_name);
 extern rwlock_t __ip_vs_svc_lock;
 extern atomic_t ip_vs_dropentry;
 extern struct ip_vs_service *
-ip_vs_lookup_service(__u32 fwmark, __u16 protocol, __u32 vaddr, __u16 vport);
-extern struct ip_vs_service * ip_vs_lookup_svc_fwm(__u32 fwmark);
+ip_vs_lookup_service(unsigned long fwmark,
+                     __u16 protocol, __u32 vaddr, __u16 vport);
+extern struct ip_vs_service * ip_vs_lookup_svc_fwm(unsigned long fwmark);
 extern struct ip_vs_dest *
 __ip_vs_lookup_real_service(__u16 protocol, __u32 daddr, __u16 dport);
 extern void update_defense_level(void);
