@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.3 2000/07/06 14:57:09 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.4 2000/07/30 04:10:05 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@iinchina.net>
  *              Peter Kese <peter.kese@ijs.si>
@@ -45,6 +45,9 @@
  */
 struct list_head *ip_vs_table;
 
+/*  SLAB cache for IPVS connections */
+static kmem_cache_t *ip_vs_conn_cachep;
+
 /*
  *  No client port connection couter
  */
@@ -307,7 +310,8 @@ void ip_vs_conn_put(struct ip_vs_conn *cp)
 	if (atomic_read(&cp->refcnt) == 1) {
 		__ip_vs_set_expire(cp, cp->timeout);
 	} else {
-		IP_VS_DBG(0, "did not set timer with refcnt=%d, called from %p\n",
+		IP_VS_DBG(0, "did not set timer with refcnt=%d, "
+                          "called from %p\n",
                           atomic_read(&cp->refcnt),
                           __builtin_return_address(0));
 	}
@@ -660,7 +664,7 @@ void ip_vs_unbind_dest(struct ip_vs_conn *cp)
                  *  if nobody refers to it (refcnt=0).
                  */
                 if (atomic_dec_and_test(&dest->refcnt))
-                        kfree_s(dest, sizeof(*dest));
+                        kfree(dest);
         }
 }
 
@@ -761,7 +765,9 @@ static void ip_vs_conn_expire(unsigned long data)
 	 */
 	if (atomic_read(&cp->refcnt) == 1) {
 		ip_vs_timeout_detach(cp);
-		kfree_s(cp, sizeof(*cp));
+                if (cp->flags & IP_VS_CONN_F_NO_CPORT)
+                        atomic_dec(&ip_vs_conn_no_cport_cnt);
+		kmem_cache_free(ip_vs_conn_cachep, cp);
 		/*  sysctl_ip_always_defrag--; */
 		MOD_DEC_USE_COUNT;
 		goto expire_out;
@@ -787,7 +793,7 @@ ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
 {
         struct ip_vs_conn *cp;
 
-        cp = kmalloc(sizeof(struct ip_vs_conn), GFP_ATOMIC);
+	cp = kmem_cache_alloc(ip_vs_conn_cachep, GFP_ATOMIC);
         if (cp == NULL) {
                 if (net_ratelimit())
                         IP_VS_ERR("ip_vs_conn_new: no memory available.\n");
@@ -966,8 +972,8 @@ void ip_vs_random_dropentry(void)
 			IP_VS_DBG(1, "Drop connection\n");
 			ct = cp->control;
 			fn = (cp->timer).function;
-			del_sltimer(&cp->timer);
-			fn((unsigned long)cp);
+			if (!del_sltimer(&cp->timer))
+				fn((unsigned long)cp);
 			if (ct && !atomic_read(&ct->n_control)) {
 				IP_VS_DBG(1, "Drop connection template\n");
 				del_sltimer(&ct->timer);
@@ -991,6 +997,16 @@ int ip_vs_conn_init(void)
         if (!ip_vs_table) {
                 return -ENOMEM;
         }
+
+        /* Allocate ip_vs_conn slab cache */
+        ip_vs_conn_cachep = kmem_cache_create("ip_vs",
+                                              sizeof(struct ip_vs_conn), 0,
+                                              SLAB_HWCACHE_ALIGN, NULL, NULL);
+        if (!ip_vs_conn_cachep) {
+                vfree(ip_vs_table);
+                return -ENOMEM;
+        }
+        
         IP_VS_INFO("Connection hash table configured "
                    "(size=%d, memory=%ldKbytes)\n",
                    IP_VS_CONN_TAB_SIZE,
@@ -1013,6 +1029,8 @@ int ip_vs_conn_init(void)
 
 void ip_vs_conn_cleanup(void)
 {
+	/* Release the empty cache */
+	kmem_cache_destroy(ip_vs_conn_cachep);
         proc_net_remove("ip_vs_conn");
         vfree(ip_vs_table);
 }
