@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.5 2000/07/06 14:57:09 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.6 2000/07/30 04:10:05 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@iinchina.net>
  *              Peter Kese <peter.kese@ijs.si>
@@ -75,7 +75,7 @@ EXPORT_SYMBOL(ip_vs_get_debug_level);
  *		0  if other IP proto
  *		-1 if error
  */
-static __inline__ int proto_doff(unsigned proto, char *th, unsigned size)
+static inline int proto_doff(unsigned proto, char *th, unsigned size)
 {
 	int ret = -1;
 	switch (proto) {
@@ -483,20 +483,6 @@ int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 	size = ntohs(iph->tot_len) - (iph->ihl * 4);
 	doff = proto_doff(iph->protocol, h.raw, size);
 
-        /*
-         *  Check if it is no_cport connection (VS/NAT only)...
-         */
-        if (cp->flags & IP_VS_CONN_F_NO_CPORT ) {
-                atomic_dec(&ip_vs_conn_no_cport_cnt);
-                ip_vs_conn_unhash(cp);
-                cp->flags &= ~IP_VS_CONN_F_NO_CPORT;
-                cp->cport = h.portp[0];
-                /* hash on new dport */
-                ip_vs_conn_hash(cp);
-
-                IP_VS_DBG(1, "filled cport=%d\n", ntohs(cp->dport));
-        }
-                
         /*
          *  Must do TCP/UDP checksum checking before mangling the packet
          */
@@ -524,6 +510,20 @@ int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
                 }
         }
         
+        /*
+         *  Check if it is no_cport connection (VS/NAT only)...
+         */
+        if (cp->flags & IP_VS_CONN_F_NO_CPORT ) {
+                atomic_dec(&ip_vs_conn_no_cport_cnt);
+                ip_vs_conn_unhash(cp);
+                cp->flags &= ~IP_VS_CONN_F_NO_CPORT;
+                cp->cport = h.portp[0];
+                /* hash on new dport */
+                ip_vs_conn_hash(cp);
+
+                IP_VS_DBG(1, "filled cport=%d\n", ntohs(cp->dport));
+        }
+                
         if (ip_route_output(&rt, cp->daddr, 0, RT_TOS(tos), 0)) {
 		IP_VS_DBG(0, "ip_vs_nat_xmit(): route error, dest: %08X\n",
                           cp->daddr);
@@ -602,6 +602,9 @@ int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
         IP_VS_DBG(1, "NAT to %u.%u.%u.%u:%d\n",
                   NIPQUAD(iph->daddr), ntohs(h.portp[1]));
 
+        /* Fixme: when application helper enlarges the packet and the length
+           is larger than the MTU of outgoing device, there will be still
+           MTU problem. */
         ip_send(skb);
 
         LeaveFunction(1);
@@ -740,6 +743,14 @@ int ip_vs_tunnel_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 }
 
 
+#if 0000
+/* Need this wrapper because NF_HOOK takes the function address */
+static inline int do_ip_send(struct sk_buff *skb)
+{
+        return ip_send(skb);
+}
+#endif
+
 /*
  *      Direct Routing
  */
@@ -777,6 +788,11 @@ int ip_vs_dr_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 
         ip_send(skb);
 
+#if 0000
+        skb->nf_debug = 0;
+        NF_HOOK(PF_INET, NF_IP_LOCAL_OUT, skb, NULL, rt->u.dst.dev,
+                do_ip_send);
+#endif
         LeaveFunction(1);
 	return 0;
 
@@ -974,15 +990,8 @@ static unsigned int ip_vs_out(unsigned int hooknum,
 		return NF_DROP;
 	}
 
-	switch (iph->protocol) {
-	case IPPROTO_ICMP:
+	if (iph->protocol == IPPROTO_ICMP)
 		return ip_vs_out_icmp(skb_p);
-	case IPPROTO_UDP:
-	case IPPROTO_TCP:
-		break;
-	default:
-		return NF_ACCEPT;
-	}
 
         IP_VS_DBG(3, "O-pkt: %s size=%d\n",
                   vs_proto_name(iph->protocol), size);
@@ -1216,7 +1225,9 @@ int ip_vs_in_icmp(struct sk_buff **skb_p)
                   NIPQUAD(ciph->saddr), ntohs(pptr[0]),
                   NIPQUAD(ciph->daddr), ntohs(pptr[1]));
 
-        /* send the mangled packet immediately here */
+        /*
+         * Send the mangled packet immediately here
+         */
         if (ip_route_output(&rt, cp->daddr, 0, RT_TOS(tos), 0)) {
                 IP_VS_DBG(0, "ip_vs_in_imcp(): route error, dest: %08X\n",
                           cp->daddr);
@@ -1301,15 +1312,8 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 		return NF_DROP;
 	}
 
-	switch (iph->protocol) {
-	case IPPROTO_ICMP:
+	if (iph->protocol == IPPROTO_ICMP)
 		return(ip_vs_in_icmp(skb_p));
-	case IPPROTO_TCP:
-	case IPPROTO_UDP:
-		break;
-	default:
-		return NF_ACCEPT;
-	}
 
 	IP_VS_DBG(2, "Incoming %s %u.%u.%u.%u:%d->%u.%u.%u.%u:%d\n",
 		  vs_proto_name(iph->protocol),
@@ -1369,29 +1373,6 @@ static unsigned int ip_vs_in(unsigned int hooknum,
                 return ret;
 	}
 
-#if 0000
-	/*
-          This code is going to die!
-          Warning, svc==0 !
-          TCP packets with SYN=0 from our svc => deliver them
-          locally (don't slowdown the packets not for our services).
-          Let the local host reject them.
-          LVS TCP with SYN=1 are rejected from ip_vs_leave
-        */
-
-	if (svc) {
-		/*
-		 * Drop packet if it is for virtual service but no entry
-		 * is found or created. Furthermore, send DEST_UNREACH icmp
-		 * packet to clients if it is not RST or it is not TCP.
-		 */
-		if (!h.th->rst || iph->protocol != IPPROTO_TCP) {
-			icmp_send(skb, ICMP_DEST_UNREACH,ICMP_PORT_UNREACH, 0);
-		}
-		return NF_DROP;
-	}
-#endif
-
 	/* sorry, all this trouble for a no-hit :) */
         IP_VS_DBG(3, "packet for %s %d.%d.%d.%d:%d continue traversal "
                   "as normal.\n",
@@ -1403,7 +1384,7 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 }
 
 
-/* Before packet filtering, forward packet through VS/DR, VS/TUN,
+/* After packet filtering, forward packet through VS/DR, VS/TUN,
    or VS/NAT(change destination) */
 static struct nf_hook_ops ip_vs_in_ops = {
         { NULL, NULL },
