@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.3 2000/07/06 14:57:09 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.4 2000/07/30 04:10:05 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@iinchina.net>
  *              Peter Kese <peter.kese@ijs.si>
@@ -330,7 +330,8 @@ __ip_vs_lookup_service(__u16 protocol, __u32 vaddr, __u16 vport)
 /*
  *  Lookup service by fwmark in the service table.
  */
-static __inline__ struct ip_vs_service * __ip_vs_lookup_svc_fwm(__u32 fwmark)
+static __inline__ struct ip_vs_service *
+__ip_vs_lookup_svc_fwm(unsigned long fwmark)
 {
 	unsigned hash;
 	struct ip_vs_service *svc;
@@ -355,7 +356,7 @@ static __inline__ struct ip_vs_service * __ip_vs_lookup_svc_fwm(__u32 fwmark)
 }
 
 struct ip_vs_service *
-ip_vs_lookup_service(__u32 fwmark, __u16 protocol, __u32 vaddr, __u16 vport)
+ip_vs_lookup_service(unsigned long fwmark, __u16 protocol, __u32 vaddr, __u16 vport)
 {
 	struct ip_vs_service *svc;
 
@@ -395,7 +396,7 @@ ip_vs_lookup_service(__u32 fwmark, __u16 protocol, __u32 vaddr, __u16 vport)
 
 	read_unlock(&__ip_vs_svc_lock);
 
-        IP_VS_DBG(2, "lookup_service fwm %d %s %u.%u.%u.%u:%u %s\n",
+        IP_VS_DBG(2, "lookup_service fwm %ld %s %u.%u.%u.%u:%u %s\n",
                   fwmark, vs_proto_name(protocol),
                   NIPQUAD(vaddr), ntohs(vport),
                   svc?"hit":"not hit");
@@ -541,7 +542,8 @@ ip_vs_get_trash_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
 
 	for (e=l->next; e!=l; e=e->next) {
 		dest = list_entry(e, struct ip_vs_dest, n_list);
-		IP_VS_DBG(1, "Destination %u/%u.%u.%u.%u:%u still in trash, refcnt=%d\n",
+		IP_VS_DBG(1, "Destination %lu/%u.%u.%u.%u:%u still in trash, "
+                          "refcnt=%d\n",
                           dest->vfwmark,
                           NIPQUAD(dest->addr), ntohs(dest->port),
                           atomic_read(&dest->refcnt));
@@ -560,13 +562,13 @@ ip_vs_get_trash_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
 		 * Try to purge the destination from trash if not referenced
 		 */
 		if (atomic_read(&dest->refcnt) == 1) {
-			IP_VS_DBG(1, "Removing destination %u/%u.%u.%u.%u:%u "
+			IP_VS_DBG(1, "Removing destination %lu/%u.%u.%u.%u:%u "
                                   "from trash\n",
 				  dest->vfwmark,
 				  NIPQUAD(dest->addr), ntohs(dest->port));
 			e = e->prev;
 			list_del(&dest->n_list);
-			kfree_s(dest, sizeof(*dest));
+			kfree(dest);
 		}
 	}
 
@@ -693,7 +695,7 @@ int ip_vs_add_dest(struct ip_vs_service *svc, struct ip_vs_rule_user *ur)
 	dest = ip_vs_get_trash_dest(svc, daddr, dport);
 	if (dest != NULL) {
 		IP_VS_DBG(1, "Get destination %u.%u.%u.%u:%u from trash, "
-			"refcnt=%d, service %u/%u.%u.%u.%u:%u\n",
+			"refcnt=%d, service %lu/%u.%u.%u.%u:%u\n",
 			NIPQUAD(daddr), ntohs(dport),
 			atomic_read(&dest->refcnt),
 			dest->vfwmark,
@@ -802,7 +804,7 @@ void __ip_vs_del_dest(struct ip_vs_dest *dest)
 	 *  the destination into the trash.
 	 */
 	if (atomic_dec_and_test(&dest->refcnt))
-		kfree_s(dest, sizeof(*dest));
+		kfree(dest);
 	else {
 		IP_VS_DBG(1, "Moving dest %u.%u.%u.%u:%u into trash, refcnt=%d\n",
                           NIPQUAD(dest->addr), ntohs(dest->port),
@@ -957,7 +959,7 @@ int ip_vs_add_service(struct ip_vs_rule_user *ur)
 	__MOD_DEC_USE_COUNT(sched->module);
 	MOD_DEC_USE_COUNT;
 	if (svc) {
-		kfree_s(svc, sizeof(struct ip_vs_service));
+		kfree(svc);
 	}
 	return ret;
 }
@@ -1060,7 +1062,7 @@ int __ip_vs_del_service(struct ip_vs_service *svc)
         /*
 	 * Free the service
 	 */
-	kfree_s(svc, sizeof(struct ip_vs_service));
+	kfree(svc);
 	MOD_DEC_USE_COUNT;
 	return 0;
 }
@@ -1338,7 +1340,7 @@ static int ip_vs_get_info(char *buf, char **start, off_t offset, int length)
                 l = &ip_vs_svc_fwm_table[idx];
                 for (e=l->next; e!=l; e=e->next) {
                         svc = list_entry(e, struct ip_vs_service, f_list);
-                        size = sprintf(buf+len, "%s  %08X %s", "FWM",
+                        size = sprintf(buf+len, "%s  %08lX %s", "FWM",
                                        svc->fwmark, svc->scheduler->name);
                         len += size;
                         pos += size;
