@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_sched.c,v 1.3 2000/07/06 14:57:09 wensong Exp $
+ * Version:     $Id: ip_vs_sched.c,v 1.4 2000/07/30 04:10:05 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@iinchina.net>
  *              Peter Kese <peter.kese@ijs.si>
@@ -96,17 +96,6 @@ int unregister_ip_vs_scheduler(struct ip_vs_scheduler *scheduler)
 		return -EINVAL;
 	}
 
-#if 0
-	/*
-	 *	Only allow unregistration if it is not referenced
-	 */
-	if (atomic_read(&scheduler->refcnt))  {
-		IP_VS_ERR("unregister_ip_vs_scheduler(): is in use by %d guys. failed\n",
-			  atomic_read(&scheduler->refcnt));
-		return -EINVAL;
-	}
-#endif
-
 	write_lock_bh(&__ip_vs_sched_lock);
 	if (scheduler->n_list.next == &scheduler->n_list) {
 		write_unlock_bh(&__ip_vs_sched_lock);
@@ -144,9 +133,6 @@ int ip_vs_bind_scheduler(struct ip_vs_service *svc,
 	}
 
 	svc->scheduler = scheduler;
-#if 0
-	atomic_inc(&scheduler->refcnt);
-#endif
 
 	if (scheduler->init_service) {
 		ret = scheduler->init_service(svc);
@@ -185,11 +171,7 @@ int ip_vs_unbind_scheduler(struct ip_vs_service *svc)
 		}
 	}
 
-#if 0
-	atomic_dec(&sched->refcnt);
-#endif
 	svc->scheduler = NULL;
-
 	return 0;
 }
 
