@@ -1,7 +1,7 @@
 /*
  * IPVS:        Weighted Round-Robin Scheduling module
  *
- * Version:     $Id: ip_vs_wrr.c,v 1.3 2000/07/06 14:57:09 wensong Exp $
+ * Version:     $Id: ip_vs_wrr.c,v 1.4 2000/07/30 04:10:05 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@iinchina.net>
  *
@@ -63,7 +63,7 @@ static int ip_vs_wrr_done_svc(struct ip_vs_service *svc)
 	/*
 	 *    Release the mark variable
 	 */
-	kfree_s(svc->sched_data, sizeof(struct ip_vs_wrr_mark));
+	kfree(svc->sched_data);
 	
 	return 0;
 }
